import abc
from pathlib import Path
import time
import typing

from requests.status_codes import codes as status_codes
from requests_toolbelt.threaded.pool import Pool

from collectionsdownload import iiifhandler


class SaveIIIFResource(abc.ABC):

    def __init__(self, iiif_resource: iiifhandler.IIIFResource, folder: Path):
        self.iiif_resource = iiif_resource
        self.folder = folder
        self.resource_id = iiif_resource.resource_id
        self.filename = self.resource_id.replace('/', '|')
        self.path = self.folder.joinpath(self.iiif_resource.barcode)
        self.path.mkdir(exist_ok=True, parents=True)
        self.path = self.path.joinpath(self.filename)

    @abc.abstractmethod
    def save(self):
        pass


class SaveIIIFTextResource(SaveIIIFResource):

    def __init__(self, iiif_resource: iiifhandler.IIIFOCRResource, folder: Path):
        super().__init__(iiif_resource, folder)
        self.iiif_resource = iiif_resource  # so the ide gets it

    def save(self):
        with self.path.open('w') as file:
            file.write(self.iiif_resource.text)


class SaveIIIFImageResource(SaveIIIFResource):

    def __init__(self, iiif_resource: iiifhandler.IIIFImageResource, folder: Path):
        super().__init__(iiif_resource, folder)
        self.iiif_resource = iiif_resource

    def save(self):
        with self.path.open('wb') as file:
            file.write(self.iiif_resource.bytes)


class CrawlCollection:

    def __init__(self, datafolder: Path, iiifcollection: iiifhandler.IIIFCollection):
        self.iiifcollection = iiifcollection
        self.ocr_datafolder = datafolder.joinpath('ocrs')
        self.ocr_datafolder.mkdir(exist_ok=True, parents=True)
        self.images_datafolder = datafolder.joinpath('images')
        self.images_datafolder.mkdir(exist_ok=True, parents=True)
        self.response_log = []
        self.counts = {}
        self.text_file_log = []
        self.image_file_log = []

        self.download_images = True
        self.download_ocrs = True

        self.limit_ocrs_per_presentation = -2
        self.limit_images_per_presentation = -2

        self.skip_ocrs_per_presentation = 0
        self.skip_images_per_presentation = 0

        # prepare iteration stuff
        self.presentation_request_pool = Pool.from_urls(
            [presentation_id for presentation_id in self.iiifcollection.presentation_ids])
        self.presentation_request_pool.join_all()
        self.presentation_responses = self.presentation_request_pool.responses()

        # Only usefull in iteration ( Crawl)

        ## presentation layer
        self.presentation_response = None
        self.iiifpresentation = None
        self.presentation_id = None
        self.current_ocr_path = None
        self.current_image_path = None
        ### ocr layer
        self.ocr_id_resource_mapping = None
        self.ocr_request_pool = None
        self.ocr_response = None
        self.iiiocrfresource = None
        self.iiiftextresource_saver = None
        ### image layer
        self.image_id_resource_mapping = None
        self.image_resource_pool = None
        self.image_response = None
        self.iiiimageresource = None
        self.iiifimagesaver = None

    def crawl(self) -> typing.Generator[
        typing.Union[iiifhandler.IIIFPresentation, SaveIIIFTextResource, SaveIIIFImageResource], None, None]:
        for self.presentation_response in self.presentation_responses:
            self.response_log.append(
                {'url': self.presentation_response.url, 'status_code': self.presentation_response.status_code,
                 'time': time.time(), 'type': 'presentation'})
            if self.presentation_response.status_code != status_codes.ok:
                continue

            self.iiifpresentation = iiifhandler.IIIFPresentation(
                id_=self.presentation_response.url, presentation_data=self.presentation_response.json(),
                limit_ocrs=self.limit_ocrs_per_presentation, skip_first_n_ocrs=self.skip_ocrs_per_presentation,
                limit_images=self.limit_images_per_presentation, skip_first_n_images=self.skip_images_per_presentation
            )

            self.counts[self.iiifpresentation.id] = {
                'n_resources': self.iiifpresentation.n_resources,
                'n_images': self.iiifpresentation.n_images,
            }

            self.presentation_id = self.iiifpresentation.id.split('/')[-1]
            self.current_image_path = self.images_datafolder.joinpath(self.presentation_id)
            self.current_ocr_path = self.ocr_datafolder.joinpath(self.presentation_id)

            yield self.iiifpresentation

            if self.download_ocrs:
                yield from self.download_ocrs_for_current_presentation()
            if self.download_images:
                yield from self.download_images_for_current_presentation()

    def download_ocrs_for_current_presentation(self) -> typing.Generator[SaveIIIFTextResource, None, None]:
        self.ocr_id_resource_mapping = {ocr_resource.resource_id: ocr_resource for ocr_resource in
                                        self.iiifpresentation.ocr_resources}
        self.ocr_request_pool = Pool.from_urls(self.ocr_id_resource_mapping.keys())
        self.ocr_request_pool.join_all()
        for self.ocr_response in self.ocr_request_pool.responses():
            self.response_log.append(
                {'url': self.ocr_response.url, 'status_code': self.ocr_response.status_code,
                 'time': time.time(), 'type': 'ocr_resource'})
            if self.ocr_response.status_code != status_codes.ok:
                continue

            self.iiiocrfresource = self.ocr_id_resource_mapping[self.ocr_response.url]
            self.iiiocrfresource.text = self.ocr_response.text
            self.iiiftextresource_saver = SaveIIIFTextResource(self.iiiocrfresource, self.current_ocr_path)
            self.iiiftextresource_saver.save()
            self.text_file_log.append(self.iiiftextresource_saver.path)
            yield self.iiiftextresource_saver

    def download_images_for_current_presentation(self) -> typing.Generator[SaveIIIFImageResource, None, None]:
        self.image_id_resource_mapping = {image_resource.resource_id: image_resource for image_resource in
                                          self.iiifpresentation.image_resources}
        self.image_resource_pool = Pool.from_urls(self.image_id_resource_mapping.keys())
        self.image_resource_pool.join_all()
        for self.image_response in self.image_resource_pool.responses():
            self.response_log.append(
                {
                    'url': self.image_response.url, 'status_code': self.image_response.status_code,
                    'time': time.time(), 'type': 'image_resource',
                }
            )

            if self.image_response.status_code != status_codes.ok:
                continue
            self.iiiimageresource = self.image_id_resource_mapping[self.image_response.url]
            self.iiiimageresource.bytes = self.image_response.content
            self.iiifimagesaver = SaveIIIFImageResource(self.iiiimageresource, self.current_image_path)
            self.iiifimagesaver.save()
            self.image_file_log.append(self.iiifimagesaver.path)
            yield self.iiifimagesaver


class CrawlMultipleCollections:

    currentCollectionCrawler: typing.Optional[CrawlCollection]

    def __init__(self, collection_urls: typing.List[str],
                 path: Path
                 ):
        self.path = path
        self.collection_urls = collection_urls
        self.collection_request_pool = None
        self.collection_jsons = None
        self.collections = None

        # cutting

        self.limit_presentations_per_collection = -2
        self.skip_presentations_per_collection = -2

        self.limit_ocrs_per_presentation = -2
        self.limit_images_per_presentation = -2

        self.skip_ocrs_per_presentation = 0
        self.skip_images_per_presentation = 0

        # what?
        self.download_images = True
        self.download_ocrs = True

        # remember

        self.log = {}
        self.counts = {}

    def prepare(self):
        """
        prepare for run!
        :return:
        """
        # prepare running
        self.collection_request_pool = Pool.from_urls(self.collection_urls)
        self.collection_request_pool.join_all()
        self.collection_jsons = [response.json() for response in self.collection_request_pool.responses()]
        self.collections = {
            collection_data['@id']: iiifhandler.IIIFCollection(collection_data,
                                                               skip_first_n_presentations=self.skip_presentations_per_collection,
                                                               limit_presentations=self.limit_presentations_per_collection
                                                               )
            for collection_data in self.collection_jsons
        }

        self.currentCollectionCrawler = None

    def run(self):
        for id_, collection in self.collections.items():
            path = self.path.joinpath(id_.split('/')[-1])
            collectionCawler = CrawlCollection(datafolder=path,iiifcollection=collection)
            self.counts[id_] = collectionCawler.counts
            collectionCawler.download_ocrs = self.download_ocrs
            collectionCawler.download_images = self.download_images
            collectionCawler.skip_ocrs_per_presentation = self.skip_ocrs_per_presentation
            collectionCawler.skip_images_per_presentation = self.skip_images_per_presentation
            collectionCawler.limit_ocrs_per_presentation = self.limit_ocrs_per_presentation
            collectionCawler.limit_images_per_presentation = self.limit_images_per_presentation

            self.currentCollectionCrawler = collectionCawler

            for any in self.currentCollectionCrawler.crawl():
                yield any

            self.log[id_] = {
                'images': [str(p) for p in collectionCawler.image_file_log], # these are paths
                'ocrs': [str(p) for p in collectionCawler.text_file_log], # these are paths
                'responses': collectionCawler.response_log
            }

            yield self.currentCollectionCrawler.iiifcollection


