"""
Handle all iiif parsing
"""
import abc
import typing


class IIIFCollection:
    """Parses manifest ids_"""

    def __init__(self, collection_data: dict, limit_presentations: typing.Optional[int] = None, skip_first_n_presentations: int = 0):
        """

        :param collection_data: the data from a request to a collection lik
        :param limit_presentations:
        :param skip_first_n_presentations:
        """
        self.index = -1
        self.limit = limit_presentations
        self.skip_items = skip_first_n_presentations

        self.collection_data = collection_data
        # if this above does not work, most scripts should fail immediately
        self.manifests = self.collection_data['manifests']
        self._manifests_iterator = iter(self.manifests)

        # fast forward with skip items
        if not (self.skip_items is None):
            for _ in range(self.skip_items):
                # keep index untouched
                self._manifests_iterator.__next__()

    @property
    def presentation_ids(self) -> typing.Generator[str, None, None]:
        for manifest in self._manifests_iterator:
            self.index += 1
            if not(self.index is None) and (self.index == self.limit):
                return
            if '@id' in manifest:
                yield manifest['@id']


class IIIFResource(abc.ABC):

    def __init__(self, barcode:str, sequence_n: str, canvas_n: str, content_n: str, resource_n: str, resource_id: str):
        """

        :param sequence_n:
        :param canvas_n:
        :param content_n: als image n
        :param resource_n:
        :param resource_id:
        """
        self.barcode = barcode
        self.resource_id = resource_id
        self.file = self.resource_id.split('/')[-1]
        self.resource_n = resource_n
        self.content_n = content_n
        self.canvas_n = canvas_n
        self.sequence_n = sequence_n


class IIIFOCRResource(IIIFResource):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._text = None                # set later

    @property
    def text(self) -> str:
        if self._text is None:
            raise KeyError('no text set')
        else:
            return self._text

    @text.setter
    def text(self, text: str) -> typing.NoReturn:
        self._text = text


class IIIFImageResource(IIIFResource):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._bytes = None                # set later

    @property
    def bytes(self) -> bytes:
        if self._bytes is None:
            raise KeyError('no text set')
        else:
            return self._bytes

    @bytes.setter
    def bytes(self, bytes: bytes) -> typing.NoReturn:
        self._bytes = bytes


class IIIFPresentation:
    """
    yields IIIF Text Resources from a presentation
    """

    def __init__(self, id_: str, presentation_data: dict,
                 limit_ocrs: typing.Optional[int] = None, skip_first_n_ocrs: typing.Optional[int] = 0,
                 limit_images: typing.Optional[int] = None, skip_first_n_images: int = 0
                 ):
        """

        :param id_:
        :param presentation_data:
        :param limit_ocrs:
        :param skip_first_n_ocrs:
        """
        self.ocr_index = -1
        self.image_index = -1
        self.id = id_
        self.presentation_data = presentation_data
        self.limit_ocrs = limit_ocrs
        self.skip_first_n_ocrs = skip_first_n_ocrs
        self.limit_images = limit_images
        self.skip_first_n_images = skip_first_n_images
        self.barcode = self.get_meta_data_by_label('Barcode')

        # count items

        self.n_images = 0
        for _ in self.image_resources:
            self.n_images += 1
        self.n_resources = 0
        for _ in self.ocr_resources:
            self.n_resources += 1

        # reset index!
        self.ocr_index = -1
        self.image_index = -1

    def get_meta_data_by_label(self, english_label):
        """Get meta_data by label
        """

        result = None

        for index0, meta_datum in enumerate(self.presentation_data['metadata']):
            if 'label' not in meta_datum:
                continue
            labels = meta_datum['label']
            correct_label = False
            if type(labels) is list:
                for index1, label_dict in enumerate(labels):
                    if label_dict['@language'] == 'en' and label_dict['@value'] == english_label:
                        correct_label = True
                        result = meta_datum['value']
                        break
            elif type(meta_datum) is dict:

                if meta_datum['label'] == english_label:
                    correct_label = True
                    result = meta_datum['value']
            else:
                # Throw some warning?
                pass

            if correct_label is True:
                break

        return result

    @property
    def ocr_resources(self) -> typing.Generator[IIIFOCRResource, None, None]:
        """honestly WTF :-)
            Maybe Write a recursive function with keys to walk through?
            treated a lot of lists as dict, now have to go through each of them … recursive?
        """

        if 'sequences' in self.presentation_data:
            sequences = self.presentation_data['sequences']

            for sequence_n, sequence in enumerate(sequences):

                if 'canvases' not in sequence:
                    continue
                canvases = sequence['canvases']

                for canvas_n, canvas in enumerate(canvases):

                    if 'otherContent' not in canvas:
                        continue

                    other_content = canvas['otherContent']

                    for content_n, content in enumerate(other_content):

                        if not ('@type' in content and content['@type'] == 'sc:AnnotationList'):
                            continue

                        if 'resources' not in content:
                            continue
                        resources = content['resources']

                        for resource_n, resource in enumerate(resources):

                            if not ('@type' in resource and resource['@type'] == 'oa:Annotation'):
                                continue

                            # There is a resource in resources … Not good naming (by me?)

                            if not ('resource' in resource):
                                continue

                            resource = resource['resource']  # … … … ?!

                            if (
                                    '@type' in resource
                                    and resource['@type'] == 'dctypes:Text'
                                    and '@id' in resource
                                    and 'format' in resource
                                    and resource['format'] == 'text/plain'
                            ):

                                ns = [str(n) for n in (self.barcode, sequence_n, canvas_n, content_n, resource_n)]

                                self.ocr_index += 1
                                if self.ocr_index < self.skip_first_n_ocrs:
                                    continue
                                if not (self.limit_ocrs is None) \
                                   and (self.ocr_index == (self.limit_ocrs + self.skip_first_n_ocrs)):
                                    return

                                yield IIIFOCRResource(*ns, resource['@id'])

                            else:
                                continue

    @property
    def image_resources(self) -> typing.Generator[IIIFImageResource, None, None]:
        """
        This partially the same as above … :-(
        :return:
        """
        if 'sequences' in self.presentation_data:
            sequences = self.presentation_data['sequences']

            for sequence_n, sequence in enumerate(sequences):

                if 'canvases' not in sequence:
                    continue
                canvases = sequence['canvases']

                for canvas_n, canvas in enumerate(canvases):

                    # here it changes
                    if 'images' not in canvas:
                        continue
                    images = canvas['images']

                    for image_n, image in enumerate(images):

                        if 'resource' not in image:
                            continue

                        resource = image['resource']

                        if '@id' not in resource:
                            continue
                        ns = [str(n) for n in (self.barcode, sequence_n, canvas_n, image_n, 0)]

                        self.image_index += 1
                        if self.image_index < self.skip_first_n_images:
                            continue
                        if not (self.limit_images is None) and (self.image_index == (self.limit_images + self.skip_first_n_images)):
                            return

                        yield IIIFImageResource(*ns, resource_id= resource['@id'])


