import json
import pathlib
import time
import typing

import click

from collectionsdownload.downloader import CrawlMultipleCollections


@click.command()
@click.argument('collections', nargs=-1, type=click.STRING)
@click.option('--images', default=False, type=click.BOOL, help='Whether or not to download the images')
@click.option('--ocrs', default=False, type=click.BOOL, help='Whether or not to download ocrs')
@click.option('--data-folder', default='./data', type=click.STRING,
              help='Where you would like to store your downloads. Will be created if does not exist.')
@click.option('--limit-presentations', default=None, type=click.INT,
              help='Limit the number of presentations downloaded per collection')
@click.option('--log-file', default=None, type=click.STRING, help='If given, will store a log at this filename')
@click.option('--count-file', default=None, type=click.STRING,
              help='if given, will store the counts of downloads at this filename')
def crawl_collections(collections: tuple, images: bool, ocrs: bool, data_folder: str,
                      limit_presentations: typing.Optional[int] = None,
                      log_file: typing.Optional[str] = None, count_file: typing.Optional[str] = None):
    """
    Download collections. Valid uris of iiif-collections. See https://iiif.io/api/presentation/3.0/#51-collection
    """
    collections = list(collections)  # it is the way it comes in, CrawlMultipleCollections thinks of lists

    try:
        download_folder = pathlib.Path(data_folder)
        download_folder.mkdir(parents=True, exist_ok=True)
    except Exception as e:
        click.echo('Could not find/create data_folder: ' + str(e))
        return

    ## Download
    click.echo('Starting download at ' + time.ctime())

    collections_crawler = CrawlMultipleCollections(collections, download_folder)
    collections_crawler.download_images = images
    collections_crawler.download_ocrs = ocrs
    collections_crawler.limit_presentations_per_collection = limit_presentations
    collections_crawler.prepare()
    _index = 0
    for _ in collections_crawler.run():
        _index += 1
        if (_index % 50) == 0:
            print(_index, time.ctime())

    if log_file:
        with open(download_folder.joinpath(log_file), 'w') as file:
            json.dump(collections_crawler.log, file)

    if count_file:
        with open(download_folder.joinpath(count_file), 'w') as file:
            json.dump(collections_crawler.counts, file)


if __name__ == '__main__':
    crawl_collections()
