import imghdr
import mimetypes
from pathlib import Path, PosixPath

import pytest
from requests.status_codes import codes as status_codes
from requests_toolbelt.threaded import pool

from collectionsdownload.downloader import CrawlCollection, CrawlMultipleCollections, SaveIIIFImageResource, SaveIIIFTextResource
from collectionsdownload.iiifhandler import IIIFCollection, IIIFPresentation


class TestCrawlCollection():

    def setup_class(self):
        """
        Threading for on erequest, because, I do not know, we will do it like this later!
        :return:
        """
        self.collection_id = 'https://iiif.onb.ac.at/presentation/collection/Karlstadt'
        request_pool = pool.Pool.from_urls([self.collection_id])
        request_pool.join_all()
        self.response_jsons = [response.json() for response in request_pool.responses()]

    def _create_crawler(self, tmp_path):
        self.iiifCollection = IIIFCollection(self.response_jsons[0], limit_presentations=1,
                                             skip_first_n_presentations=0)
        self.collectionCrawler = CrawlCollection(tmp_path, self.iiifCollection)
        self.collectionCrawler.limit_images_per_presentation = 1
        self.collectionCrawler.limit_ocrs_per_presentation = 1

    @pytest.fixture(scope='session')
    def tmp_path(self, tmpdir_factory):
        stupid_local_path = tmpdir_factory.mktemp('temp')
        return Path(str(stupid_local_path))

    def test_download_ocrs(self, tmp_path):
        self._create_crawler(tmp_path)
        self.collectionCrawler.download_images = False
        self.collectionCrawler.download_ocrs = True
        crawler = self.collectionCrawler.crawl()
        assert next(crawler).__class__ is IIIFPresentation
        assert next(crawler).__class__ is SaveIIIFTextResource
        with pytest.raises(StopIteration) as _:
            next(crawler)

        assert self.collectionCrawler.response_log.__len__() == 2  # a manifest, an image
        assert self.collectionCrawler.response_log[0]['status_code'] == status_codes.ok
        assert self.collectionCrawler.response_log[0]['type'] == 'presentation'
        assert self.collectionCrawler.response_log[1]['type'] == 'ocr_resource'

        assert self.collectionCrawler.text_file_log.__len__() == 1
        path = self.collectionCrawler.text_file_log[0]
        assert path.__class__ == PosixPath
        path = Path(path)  # ide …
        assert path.exists()
        assert mimetypes.guess_type(str(path.absolute()))[0] == 'text/plain'

    def test_download_images(self, tmp_path):
        self._create_crawler(tmp_path)
        self.collectionCrawler.download_images = True
        self.collectionCrawler.download_ocrs = False
        crawler = self.collectionCrawler.crawl()
        presentation = next(crawler)
        assert presentation.__class__ is IIIFPresentation
        assert presentation.id == 'https://iiif.onb.ac.at/presentation/ABO/Z169360503/manifest/'
        resource = next(crawler)
        assert resource.__class__ is SaveIIIFImageResource
        assert resource.resource_id == 'https://iiif.onb.ac.at/images/ABO/Z169360503/00000001/full/full/0/native.jpg'
        with pytest.raises(StopIteration) as _:
            next(crawler)

        assert self.collectionCrawler.response_log.__len__() == 2  # a manifest, an image
        assert self.collectionCrawler.response_log[0]['status_code'] == status_codes.ok
        assert self.collectionCrawler.response_log[0]['type'] == 'presentation'
        assert self.collectionCrawler.response_log[1]['type'] == 'image_resource'

        assert self.collectionCrawler.image_file_log.__len__() == 1
        path = self.collectionCrawler.image_file_log[0]
        assert path.__class__ == PosixPath
        path = Path(path)  # ide …
        assert path.exists()
        assert not (imghdr.what(path) is None)

    def test_crawl(self, tmp_path):
        self._create_crawler(tmp_path)
        self.collectionCrawler.download_images = True
        self.collectionCrawler.download_ocrs = True
        crawler = self.collectionCrawler.crawl()
        assert next(crawler).__class__ is IIIFPresentation
        n = next(crawler)
        assert (n.__class__ is SaveIIIFTextResource) or (n.__class__ is SaveIIIFImageResource)
        next(crawler)
        with pytest.raises(StopIteration) as _:
            next(crawler)


class TestCrawlMultipleCollections:

    @pytest.fixture(scope='session')
    def tmp_path(self, tmpdir_factory):
        stupid_local_path = tmpdir_factory.mktemp('temp')
        return Path(str(stupid_local_path))

    def test_run_images(self, tmp_path):
        """
        This is basically the script to run finally (without zip)
        :return:
        """
        collection_id = 'https://iiif.onb.ac.at/presentation/collection/Karlstadt'
        crawlMultipleCollections = CrawlMultipleCollections([collection_id], tmp_path)
        crawlMultipleCollections.download_ocrs = False
        crawlMultipleCollections.download_images = True
        crawlMultipleCollections.limit_presentations_per_collection = 1
        crawlMultipleCollections.limit_images_per_presentation = 1
        crawlMultipleCollections.prepare()
        runner = crawlMultipleCollections.run()
        assert next(runner).__class__ is IIIFPresentation
        assert crawlMultipleCollections.currentCollectionCrawler.__class__ is CrawlCollection
        assert next(runner).__class__ is SaveIIIFImageResource
        assert next(runner).__class__ is IIIFCollection
        with pytest.raises(StopIteration) as _:
            next(runner)

        assert crawlMultipleCollections.log[collection_id]['responses'].__len__() is 2
        assert crawlMultipleCollections.log[collection_id]['images'].__len__() is 1
        assert crawlMultipleCollections.log[collection_id]['images'][0].__class__ is str
        path = Path(crawlMultipleCollections.log[collection_id]['images'][0])
        assert path.exists()
        assert not (imghdr.what(path) is None)



